@@ -281,7 +281,7 @@ def _setup_flask_config(run_config: RuntimeConfig, dm: Dimensigon):
 
     dm.config.flask_conf = flask_config
 
-    result = dm.engine.execute(Dimension.__table__.select(Dimension.current).where(Dimension.current == True))
+    result = dm.engine.execute(Dimension.__table__.select().where(Dimension.current == True))
     result = result.fetchall()
     if len(result) == 1:
         dm.config.flask_conf.SECRET_KEY = result[0][0]
